@@ -25,7 +25,13 @@
 
   http://corp.avantgo.com/mal/
 
+
 */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 /* Thu, 29 Jul 1999 12:43:01 +0200
  *     Turbo Fredriksson <turbo@nocrew.org>
  *     Added daemon mode (close STDIO and disconnect from the shell
@@ -47,20 +53,6 @@
 #include <getopt.h>
 #endif
 
-#include <AGNet.h>
-#include <AGUtil.h>
-#include <AGClientProcessor.h>
-#include <AGProtocol.h>
-#include <AGBufferReader.h>
-#include <AGPalmProtocol.h>
-#include <AGUserConfig.h>
-#include <AGServerConfig.h>
-#include <AGSyncCommon.h>
-#include <AGCommandProcessor.h>
-#include <AGDesktopInfoPalm.h>
-#include <AGTimeoutMessageBox.h>
-#include <AGMobileLinkSettings.h>
-#include <MAL31UserConfig.h>
 
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -69,6 +61,8 @@
 #include <pi-version.h>
 
 #include <gpilotd/gnome-pilot-conduit.h>
+#include "malsync.h"
+
 extern GnomePilotConduit *the_conduit;
 
 #define DEFAULT_CARD_NUM 0
@@ -90,32 +84,6 @@ static char *progname       = NULL;
 #define debug_print(x)
 #endif
 
-typedef struct {
-
-    AGDeviceInfo * deviceInfo;
-    AGUserConfig * userConfig;
-    AGServerConfig * currentServerConfig;
-    AGClientProcessor * clientProcessor;
-    AGPlatformCalls * platform;
-    AGRecord * record;
-    AGDBConfig * currentDb;
-    AGCommandProcessor *commandProcessor;
-    AGBool quit;
-    
-    /* Pilot-specific */
-    AGBool currentDBIsResourceType;
-    int pilot_rHandle;
-    int pilot_RecIndex;
-    recordid_t id;
-    uint8  *pilot_buffer;
-    int pilot_buffer_size;
-    
-    /* Secure Network Library Stuff */
-    AGBool          hasseclib;
-
-} PalmSyncInfo;
-
-
 #ifdef HAVE_GETOPTLONG
 static struct option long_options[] =
 {
@@ -270,9 +238,13 @@ syncInfoFree(PalmSyncInfo * pInfo)
         if (NULL != pInfo->userConfig)
             AGUserConfigFree(pInfo->userConfig);
 
+#ifdef PILOT_LINK_0_12
+        if (NULL != pInfo->pi_buf)
+            pi_buffer_free (pInfo->pi_buf);
+#else
         if (NULL != pInfo->pilot_buffer)
             free(pInfo->pilot_buffer);
-
+#endif
         if (NULL != pInfo->commandProcessor)
             AGCommandProcessorFree(pInfo->commandProcessor);
 
@@ -294,10 +266,16 @@ syncInfoNew()
 
         bzero(pInfo, sizeof(PalmSyncInfo));
 
+#ifdef PILOT_LINK_0_12
+        pInfo->pi_buf = pi_buffer_new (pbs);
+        if (NULL == pInfo->pi_buf)
+             goto fail;
+#else
         pInfo->pilot_buffer_size    = pbs;
         pInfo->pilot_buffer         = (uint8 *)malloc(pbs);
         if (NULL == pInfo->pilot_buffer)
             goto fail;
+#endif
 
         /* Allocate the platform calls record. */
         pInfo->platform = (AGPlatformCalls *)malloc(sizeof(AGPlatformCalls));
@@ -345,8 +323,12 @@ int16 readInt16(uint8 * p)
 /*---------------------------------------------------------------------------*/
 static void 
 readAndUseDeviceInfoDatabase(AGDeviceInfo * devInfo,
+#ifdef PILOT_LINK_0_12
+                             pi_buffer_t *pi_buf)
+#else
                              uint8 *dev_db_info_buffer,
                              uint32 dev_db_info_buffer_size)
+#endif
 {
     int database_id = 0;
     long result;
@@ -372,14 +354,24 @@ readAndUseDeviceInfoDatabase(AGDeviceInfo * devInfo,
         recordid_t id;
         int rc;
 
+#ifdef PILOT_LINK_0_12
+        rc = dlp_ReadRecordByIndex(sd, database_id, 0,
+                                    pi_buf, 
+                                    &id,
+                                    &attr, &cat);
+#else
         rc = dlp_ReadRecordByIndex(sd, database_id, 0, 
                                    (void *)dev_db_info_buffer, 
                                    &id, &dev_db_info_buffer_size, 
                                    &attr, &cat);
-    
+#endif    
 
         if (rc >= 0) {
+#ifdef PILOT_LINK_0_12
+            uint8 *p = pi_buf->data;
+#else
             uint8 *p = dev_db_info_buffer;
+#endif
             int16 dev_db_info_version = readInt16(p);
             p+=sizeof(int16);
             devInfo->colorDepth = readInt32(p);
@@ -471,9 +463,14 @@ readDeviceInfo(PalmSyncInfo * pInfo)
                devInfo->colorDepth);
     }
 
+#ifdef PILOT_LINK_0_12
+    readAndUseDeviceInfoDatabase(devInfo,
+                                 pInfo->pi_buf);
+#else
     readAndUseDeviceInfoDatabase(devInfo,
                                  pInfo->pilot_buffer,
                                  pInfo->pilot_buffer_size);
+#endif
 
     /* Override the color depth if the user wants low res images. */
     if (lowres) {
@@ -516,6 +513,7 @@ static int createDatabase(AGDBConfig *db)
 {
     int dbhandle;
     long creator;
+    uint32 crID32, type32;
     int flags;
     int cardNo = DEFAULT_CARD_NUM;
     long type;
@@ -528,8 +526,12 @@ static int createDatabase(AGDBConfig *db)
     if (verbose)
         printf("createDatabase\n");
     
-    getPalmDatabaseCreationInfo(db, (uint32*)&creator, 
-                                (uint32*)&flags, (uint32*)&type);
+    getPalmDatabaseCreationInfo(db, &crID32, 
+                                &flags, &type32);
+
+    /* convert AG 32 bit values to long, for dlp */
+    creator = crID32;
+    type = type32;
 
     if ((err = dlp_CreateDB(sd,  creator,  type, cardNo, flags,  
                             version, db->dbname, &dbhandle)) < 0) {
@@ -664,31 +666,56 @@ static int32
 readDeviceUserConfig32(int userConfigDBHandle, AGUserConfig **deviceUserConfig)
 {
     recordid_t id;
-    int bufferSize = BUFFERSIZE;
     int attr = 0;
     int cat  = 0;
     int rc;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;    
+#else
+    int bufferSize = BUFFERSIZE;
     uint8 buffer[BUFFERSIZE];
+#endif
     AGBufferReader * r = NULL;
 
+#ifdef PILOT_LINK_0_12
+    pi_buf = pi_buffer_new (BUFFERSIZE);
+
+    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf, 
+                               &id, &attr, &cat);
+     
+#else
     rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer, 
                                &id, &bufferSize, &attr, &cat);
-    
+#endif    
     if (rc < 0) {
         if (verbose)
             printf("readDeviceUserConfig: dlp_ReadRecordByIndex , err = %d\n",
                    rc);
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free (pi_buf);
+#endif
         return 0;
     }
-    
+
+#ifdef PILOT_LINK_0_12
+    r = AGBufferReaderNew(pi_buf->data);    
+#else
     r = AGBufferReaderNew(buffer);
+#endif
     if (r) {
         *deviceUserConfig = AGUserConfigNew();
         AGUserConfigReadData(*deviceUserConfig, (AGReader*)r);
         AGBufferReaderFree(r);
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free (pi_buf);
+#endif
         return id;
-    } else
+    } else {
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free (pi_buf);
+#endif
         return 0;
+    }
 }
 #define BUFFERSIZE 0xFFFF
 /*---------------------------------------------------------------------------*/
@@ -696,24 +723,43 @@ static int32 readDeviceUserConfig31(int userConfigDBHandle,
                                     AGUserConfig **deviceUserConfig)
 {
     recordid_t id;
-    int bufferSize = BUFFERSIZE;
     int attr = 0;
     int cat  = 0;
     int rc;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;
+#else
+    int bufferSize = BUFFERSIZE;
     uint8 buffer[BUFFERSIZE];
+#endif
     AGBufferReader * r = NULL;
 
+#ifdef PILOT_LINK_0_12
+    pi_buf = pi_buffer_new (BUFFERSIZE);
+    
+    rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf, 
+                               &id, &attr, &cat);
+#else
     rc = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer, 
                                &id, &bufferSize, &attr, &cat);
+#endif
     
     if (rc < 0) {
         if (verbose)
             printf("readDeviceUserConfig: dlp_ReadRecordByIndex , err = %d\n",
                    rc);
+#ifdef PILOT_LINK_0_12
+    pi_buffer_free (pi_buf);
+#endif
         return 0;
     }
-    
+
+#ifdef PILOT_LINK_0_12
+    r = AGBufferReaderNew(pi_buf->data);    
+    pi_buffer_free (pi_buf);
+#else
     r = AGBufferReaderNew(buffer);
+#endif
     if (r) {
         *deviceUserConfig = AGUserConfigNew();
         MAL31ReadUserData(*deviceUserConfig, (AGReader*)r);
@@ -741,24 +787,35 @@ static void writeDeviceUserConfig(int userConfigDBHandle,
 {
 
     recordid_t id;
-    int bufferSize = BUFFERSIZE;
     int attr = 0;
     int cat  = 0;
+#ifdef PILOT_LINK_0_12
+    pi_buffer_t *pi_buf;
+#else
+    int bufferSize = BUFFERSIZE;
     uint8 buffer[BUFFERSIZE];
+#endif
     AGBufferWriter * w = NULL;
     w = AGBufferWriterNew(0);
     if (w) {
         long result;
         
+#ifdef PILOT_LINK_0_12
+        pi_buf = pi_buffer_new (BUFFERSIZE);
+#endif
         if (threeone) {
             MAL31WriteUserData(deviceUserConfig, (AGWriter*)w);
         } else {
             AGUserConfigWriteData(deviceUserConfig, (AGWriter*)w);
         }
 
+#ifdef PILOT_LINK_0_12
+        result = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, pi_buf, 
+                                       &id, &attr, &cat);
+#else
         result = dlp_ReadRecordByIndex(sd, userConfigDBHandle, 0, (void *)buffer, 
                                    &id, &bufferSize, &attr, &cat);
-        
+#endif        
         if (result < 0)
             id = 0;
 
@@ -768,6 +825,9 @@ static void writeDeviceUserConfig(int userConfigDBHandle,
                                   &id);
         AGBufferWriterFree(w);
 
+#ifdef PILOT_LINK_0_12
+        pi_buffer_free (pi_buf);
+#endif
     }
 }
 
@@ -1055,17 +1115,29 @@ getRecordBase(PalmSyncInfo * pInfo, AGBool modonly,  AGRecord **record,
     int32 result;
     int att = 0;
     int cat = 0;
+#ifndef PILOT_LINK_0_12
     int size = pInfo->pilot_buffer_size;
+#endif
     int idx   = pInfo->pilot_RecIndex++;
 
     result = (modonly) ?
+#ifdef PILOT_LINK_0_12
+        dlp_ReadNextModifiedRec (sd, pInfo->pilot_rHandle, pInfo->pi_buf,
+                                  &pInfo->id, &idx,
+                                 &att, &cat)
+        :
+        dlp_ReadRecordByIndex(sd, pInfo->pilot_rHandle, idx,
+                              pInfo->pi_buf, &pInfo->id,
+                              &att, &cat);
+#else
         dlp_ReadNextModifiedRec (sd, pInfo->pilot_rHandle, pInfo->pilot_buffer,
-                                 &pInfo->id, &idx,
-                                 &size, &att, &cat)
+                             &pInfo->id, &idx,
+                             &size, &att, &cat)
         :
         dlp_ReadRecordByIndex(sd, pInfo->pilot_rHandle, idx,
                               pInfo->pilot_buffer, &pInfo->id,
                               &size, &att, &cat);
+#endif
     
     if (result < 0) {
         closeDatabase(pInfo);
@@ -1079,9 +1151,15 @@ getRecordBase(PalmSyncInfo * pInfo, AGBool modonly,  AGRecord **record,
             return leaveGetRecord(pInfo, AGCLIENT_ERR);
         }
     }
+#ifdef PILOT_LINK_0_12
+     pInfo->record = AGRecordInit(pInfo->record, pInfo->id,
+                                  AGPalmPilotAttribsToMALMod((uint8)att),
+                                  pInfo->pi_buf->used, pInfo->pi_buf->data, 0, NULL);
+#else
     pInfo->record = AGRecordInit(pInfo->record, pInfo->id,
                                  AGPalmPilotAttribsToMALMod((uint8)att),
                                  size, pInfo->pilot_buffer, 0, NULL);
+#endif
 
     *record = pInfo->record;
     return AGCLIENT_CONTINUE;
@@ -1397,7 +1475,9 @@ SigHandler(int signal)
 static void 
 Connect(PalmSyncInfo *pi) 
 {
+#ifndef PILOT_LINK_0_12
     struct pi_sockaddr addr;
+#endif
     int ret;
     
     if (sd != 0)
@@ -1413,11 +1493,15 @@ Connect(PalmSyncInfo *pi)
         perror("pi_socket");
         exit(1);
     }
-    
+
+#ifdef PILOT_LINK_0_12    
+    ret = pi_bind(sd, device);
+#else
     addr.pi_family = PI_AF_PILOT;
-    strcpy(addr.pi_device, device);
-    
+    strcpy(addr.pi_device, device);    
     ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
+#endif
+
     if(ret == -1) {
         fprintf(stderr, "Unable to bind to port '%s'.\n", device);
         exit(1);
