@@ -27,11 +27,11 @@
 void AGMd5(uint8 *buf, int32 buflen, uint8 digest[16])
 {
 
-    MD5_CTX context;
+    AGMD5_CTX context;
     
-    MD5Init (&context);
-    MD5Update (&context, buf, buflen);
-    MD5Final (digest, &context);
+    AGMD5Init (&context);
+    AGMD5Update (&context, buf, buflen);
+    AGMD5Final (digest, &context);
 
 }
 
