@@ -20,6 +20,10 @@
  *          Vadim Strizhevsky
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <glib.h>
 #include <gnome.h>
 
@@ -304,6 +308,8 @@ generate_name (MemoLocalRecord *local, GnomePilotConduitStandardAbs *abs)
 	
 	fname = newfilename (local);
 	categorypath = category_path (local->local.archived?16:local->category, abs);
+	if (local->filename)
+		g_free (local->filename);
 	local->filename = g_strdup_printf ("%s/%s",
 					   categorypath,
 					   fname);
@@ -675,6 +681,9 @@ load_categories (GnomePilotConduitStandardAbs *abs)
 	categories = g_hash_table_new (g_str_hash, g_str_equal);
 	filename = g_strdup_printf ("%s/.categories", GET_CONDUIT_CFG (abs)->dir);
 
+	if (!g_file_test (filename, G_FILE_TEST_EXISTS))
+		return categories;
+
 	if ((f = fopen (filename,"r"))==NULL) return NULL;
 	
 	while (fgets (category_dir, FILENAME_MAX-1, f)!=NULL) {
@@ -718,12 +727,6 @@ free_str_foreach (gchar *s, gpointer whatever)
 	g_free (s);
 }
 
-static void
-free_loadinfo_foreach (LoadInfo *info, gpointer whatever) 
-{
-	g_free (info);
-}
-
 /** loads the records into the abs structure */
 static gboolean
 load_records (GnomePilotConduitStandardAbs *abs) 
@@ -759,16 +762,16 @@ load_records (GnomePilotConduitStandardAbs *abs)
 		if (strcmp (de->d_name,".categories")==0) continue;
 		
 		/* first load id:sec:names from the .ids file */
-		loadinfo = g_hash_table_new (g_str_hash, g_str_equal);
+		loadinfo = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 		filename = g_strdup_printf ("%s/%s", GET_CONDUIT_CFG (abs)->dir, de->d_name);
 		category = GPOINTER_TO_INT (g_hash_table_lookup (categories, filename));
+		g_free (filename);
 		if (category<0 || category > 16) category = 0;
 		if (category == 16) {
 			/* ignore archived dir */
 			continue;
 		}
-		g_free (filename);
 		
 		filename = g_strdup_printf ("%s/%s/.ids", GET_CONDUIT_CFG (abs)->dir, de->d_name);
 		if ((idfile=fopen (filename,"rt"))!=NULL) {
@@ -813,7 +816,6 @@ load_records (GnomePilotConduitStandardAbs *abs)
 						local->local.secret = info->secret;
 						local->mtime=info->mtime;
 						g_hash_table_remove (loadinfo, local->filename);
-						g_free (info);
 					} else {
 						local->local.ID = 0;
 						local->local.secret = 0;
@@ -842,7 +844,6 @@ load_records (GnomePilotConduitStandardAbs *abs)
 		if (g_hash_table_size (loadinfo)>0) {
 			deleted += g_hash_table_size (loadinfo);
 			g_hash_table_foreach (loadinfo,(GHFunc)create_deleted_record_foreach,&GET_CONDUIT_DATA (abs)->records);
-			g_hash_table_foreach (loadinfo,(GHFunc)free_loadinfo_foreach, NULL);
 		}
 		g_hash_table_destroy (loadinfo);
 	}
@@ -865,22 +866,40 @@ static gint
 pre_sync (GnomePilotConduit *c, GnomePilotDBInfo *dbi) 
 {
 	int l;
+#ifdef PILOT_LINK_0_12
+	pi_buffer_t *pi_buf;
+#else
 	unsigned char *buf;
-  
+#endif  
 	g_message ("MemoFile Conduit v %s", CONDUIT_VERSION);
 
 	LOG ("PreSync");
 
 	GET_CONDUIT_DATA (c)->dbi=dbi;
   
+#ifdef PILOT_LINK_0_12
+	pi_buf = pi_buffer_new (0xffff);
+	if ((l=dlp_ReadAppBlock (dbi->pilot_socket, dbi->db_handle, 0,0xffff, pi_buf))<0) {	
+#else
 	buf = (unsigned char*)g_malloc (0xffff);
 	if ((l=dlp_ReadAppBlock (dbi->pilot_socket, dbi->db_handle, 0,(unsigned char *)buf, 0xffff))<0) {
+#endif
 		LOG ("dlp_ReadAppBlock (...) failed");
+#ifdef PILOT_LINK_0_12
+		pi_buffer_free (pi_buf);
+#else
 		g_free (buf);
+#endif
 		return -1;
 	}
+
+#ifdef PILOT_LINK_0_12
+	unpack_MemoAppInfo (&(GET_CONDUIT_DATA (c)->ai), pi_buf->data, l);
+	pi_buffer_free (pi_buf);
+#else
 	unpack_MemoAppInfo (&(GET_CONDUIT_DATA (c)->ai), buf, l);
 	g_free (buf);
+#endif
 
 	if (GET_CONDUIT_CFG (c)->dir==NULL || *(GET_CONDUIT_CFG (c)->dir) == '\0') {
 		return -1;
@@ -1230,7 +1249,7 @@ insert_numeric_callback (GtkEditable    *editable, const gchar    *text,
 }
 
 /* so I like structures. */
-FieldInfo fields[] =
+static FieldInfo fields[] =
 { { N_("Memos directory:"), NULL,"dir", insert_ignore_space},
   { N_("Ignore start:"), NULL,"ignore_start", insert_ignore_space},
   { N_("Ignore end:"), NULL,"ignore_end", insert_ignore_space},
@@ -1272,7 +1291,8 @@ static GtkWidget
 
 	for (i=0;i<count;i++) {
 		label = gtk_label_new (_(fields[i].name));
-		gtk_table_attach (GTK_TABLE (table), label, 1, 2, i, i+1, 0, 0, 0, 0);
+		gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+                gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, i, i+1);
 		if (fields[i].label_data!=NULL) {
 			gtk_object_set_data (GTK_OBJECT (vbox), fields[i].label_data, label);
 		}
