@@ -198,7 +198,7 @@ synchronize (GnomePilotConduit *c,
 	} else {
 		t = time (NULL);
 		err = dlp_SetSysDateTime (dbi->pilot_socket, t);
-		LOG ("synchronization to PDA = %d", t, err);
+		LOG ("synchronization to PDA = %d", err);
 	}
 	
 	return err;
